;IR2 - Infrared Receiver Code
;Version 1.1



;CONFIGURABLE OPTIONS

;Transmitter type 
;uncomment the correct one, comment out the others
;#DEFINE JR
#DEFINE FUTABA
;#DEFINE SANWA

;These options will only take effect on the 16F630
#IFDEF __16F630
  #define 	MAXCHANNELS				d'4'	;number of channels to parse, 3 or 4
  #define	CHAN4ESC						;optional, treat rudder as actuator or ESC?, comment out if unwanted
  #define	CHAN4MIX						;optional, linear mix throttle into rudder channel, uscomment to use
#ELSE
  ;This option only make sense on other PICS
  ;Output Channel 4(rudder) on ESC instead of throttle (all other pins stay the same)
  ;This allows two 8-pin PICS to be used for a 4-channel heli system
  ;Burn one PIC without this option, and the other PIC with this option
  ;Uncomment to use
  ;#DEFINE FAKE4CH
  ;#define	CHAN4MIX						;optional, linear mix throttle into rudder channel, uncomment to use
#ENDIF

;offset for CHAN4MIX (if used), affects zero point of rudder when mixed
#define	MIX_OFFSET				d'1'	

;END CONFIGURABLE OPTIONS















#IFDEF __12C509A
	MESSG "Assembling for 12C509A"
	LIST	P=PIC12C509A
	INCLUDE	"P12C509A.INC"
	__CONFIG _MCLRE_OFF & _CP_OFF & _WDT_OFF & _IntRC_OSC
#ENDIF

#IFDEF __12F629
	MESSG "Assembling for 12F629"
	LIST	P=PIC12F629
	INCLUDE "P12F629.INC"
	__CONFIG _MCLRE_OFF & _CP_OFF & _WDT_OFF & _INTRC_OSC_NOCLKOUT & _CPD_OFF & _PWRTE_ON & _BODEN_ON
	;ERRORLEVEL -302					;turn off register not in bank0 warnings
#ENDIF

#IFDEF __12F675
	MESSG "Assembling for 12F675"
	LIST	P=PIC12F675
	INCLUDE "P12F675.INC"
	__CONFIG _MCLRE_OFF & _CP_OFF & _WDT_OFF & _INTRC_OSC_NOCLKOUT & _CPD_OFF & _PWRTE_ON & _BODEN_ON
	;ERRORLEVEL -302					;turn off register not in bank0 warnings
#ENDIF

#IFDEF __16F630
	MESSG "Assembling for 16F630"
	LIST	P=PIC16F630
	INCLUDE "P16F630.INC"
	__CONFIG _MCLRE_OFF & _CP_OFF & _WDT_OFF & _INTRC_OSC_NOCLKOUT & _CPD_OFF & _PWRTE_ON & _BODEN
    #DEFINE GPIO PORTA  	;Porta is equiv to GPIO
	#DEFINE TRISIO TRISA 	;Trisa is equiv to TRISIO
	;ERRORLEVEL -302					;turn off register not in bank0 warnings
#ENDIF



;DEFINE CONSTANTS
;----------------
;CYCLE_TIME MUST BE UPDATED FOR CHANGES TO THE DURATION OF DOPWM
#define		CYCLE_TIME				d'32'		;number of uSec DOPWM takes, used as timeref for rest of timing
#define		ACTUAL_CYCLE_TIME		(CYCLE_TIME + 3) ;CYCLE_TIME plus calls	
#define		TIME_VAL				((d'256' - CYCLE_TIME) + 2)	;TMR0 value to maintain consistent DOPWM time

; High to High pulses longer than this are rejected
#define 	MAX_ALLOWED_PULSE_WIDTH (d'2200'/ACTUAL_CYCLE_TIME)

#define 	MAX_MEASURED_WIDTH		(d'1950'/ACTUAL_CYCLE_TIME - 1) ;(-1 => towards centre)
#define 	MIN_PULSE_WIDTH			(d'1120'/ACTUAL_CYCLE_TIME + 1) ;(+1 => towards centre)

#define 	MIN_THROTTLE_WIDTH		(d'1200'/ACTUAL_CYCLE_TIME)
#define 	MAX_THROTTLE_WIDTH		(d'1870'/ACTUAL_CYCLE_TIME)

; make the number of counts per frame a bit smaller to make sure full on is always reachable with no trim
#define 	MAX_FRAME_COUNT			((MAX_MEASURED_WIDTH - MIN_PULSE_WIDTH) - 2)
#define 	CENTER_STICK			((d'1500'/ACTUAL_CYCLE_TIME)+1)

#define 	WAIT_17mS				(d'17000'/ACTUAL_CYCLE_TIME)
#define 	WAIT_3mS				(d'3000'/ACTUAL_CYCLE_TIME)

; High pulses shorter than this are rejected
#define 	WAIT_300uS				(d'300'/ACTUAL_CYCLE_TIME)

; The number of bad pwm steps before we switch everything off in weve had an error
;After ((256*ErrorCount2) + ErrorCount1) errors, we drop into fail safe mode
;This is about 1 second
#define		ErrorCount1				d'172'
#define		ErrorCount2				d'4'

#define		INPUT_ERROR_FLAG		OUT_BUF, 7	;Flag to indicate input data is invalid

#IFNDEF __16F630
  #define 	MAXCHANNELS				d'3'	;12XXXX family only has pins for 3 channels
#ENDIF

#define		INPUT_PIN				GPIO, 3
#define		AIL_LOW_PIN				OUT_BUF, 0
#define		AIL_HIGH_PIN			OUT_BUF, 1
#define		ESC_PIN					OUT_BUF, 2
#define		ELE_LOW_PIN				OUT_BUF, 4
#define		ELE_HIGH_PIN			OUT_BUF, 5
#define		RUD_LOW_PIN				OUT_BUFC, 0
#define		RUD_HIGH_PIN			OUT_BUFC, 1


;VARIABLES
;---------
#IFDEF __12C509A
  #define CBVAL 0x007		;12C509A only has 25 GP + 16(banked) registers, starting at 07h
#ELSE
  #define CBVAL 0x020		;12F6XX/16F6XX regs start at 20h
#ENDIF

 CBLOCK CBVAL				;Start a Variable block, starting at CBVAL set above
OUTPUT								;register to write to GPIO
OUTPUTC								;register to write to PORTC
OUT_BUF								;Manipulate this register, then copy to OUTPUT
OUT_BUFC							;Manipulate this register, then copy to OUTPUTC
BAD_INPUT_CNT						;Counter for # of bad input cycles
BAD_INPUT_CNT2						;2nd Counter for # of bad input cycles
nTimer								;Temp register for counting input pulse lengths
START_PULSE_LENGTH					;Counter for start pulse length
ESC_VAL								;ESC PWM duration value
AIL_VAL								;AILERON PWM duration value
ELE_VAL								;ELEVATOR PWM duration value
RUD_VAL								;RUDDER PWM duration value

;values received
ESC_BUF								;ESC buffer
AIL_BUF								;AILERON buffer
ELE_BUF								;Elevator buffer
RUD_BUF								;Rudder buffer
ESC_BUF_OLD							;ESC last value
AIL_BUF_OLD							;Aileron last value
ELE_BUF_OLD							;Elevator last value
RUD_BUF_OLD							;Rudder last value

;these 3 are loaded with pointers to the appropriate buffer
CH1_BUF_PTR					
CH2_BUF_PTR					
CH3_BUF_PTR					
CH4_BUF_PTR							

PWM_CNTR							;PWM counter
COUNTER								;counter for parsing channels

currentVal							;Temp variables used when parsing input
lastVal 							;
pinVal								;
pwmVal								;
 ENDC






;PROGRAM CODE STARTS HERE
 ORG 0  
  Goto SETUP

DOPWM
;START SUBROUTINE DOPWM
;Perform PWM by counting and turning off the pins after
;the specified duration
;Move new values on output pins every so often
;Shutdown outputs after no input received for a period of time
	;jump to CHECK_CHANNELS most of the time
  movf PWM_CNTR, W							;load PWN_CNTR
  xorlw MAX_FRAME_COUNT						;xor with MAX_FRAME_COUNT
  btfss STATUS, Z							;if zero (equal), skip
  goto CHECK_CHANNELS						;else goto CHECK_CHANNELS

	;after MAX_FRAME_COUNT runs, reset and load new values
  clrf PWM_CNTR								;reset PWM counter
  btfss INPUT_ERROR_FLAG					;test for input parsing errors
  goto GOOD_OUTPUTS							;inputs still good, use em
											;else, inputs are bad
  movf BAD_INPUT_CNT, W						;load counter1
  iorwf BAD_INPUT_CNT2, W					;ior with counter2
  btfss STATUS, Z							;test if they're both zero
  goto FEW_ERRORS							;if not zero, jump down
	;if we get here, we've had no good input for a while
	;so set all outputs to zero to minimize crash damage
	;and provide feedback to the pilot
  clrf OUTPUT								;clear outputs
#IF MAXCHANNELS > 3
  clrf OUTPUTC								;clear outputs 
#ENDIF
  goto WRITE_OUTPUTS


FEW_ERRORS
	;if we get here, we've had a few input errors
	;we decrease the 1st counter, and if that's zero
	;decrease the 2nd counter (causing counter1 to underflow)
  decf BAD_INPUT_CNT, f						;decrease the counter
  incf BAD_INPUT_CNT, W						;bump it back for testing into W
  btfsc STATUS, Z							;if the result is zero
  decf BAD_INPUT_CNT2, f					;decrease the 2nd counter
  goto WRITE_OUTPUTS

GOOD_OUTPUTS
	;reset the bad input counters since we received good input
  movlw ErrorCount1
  movwf BAD_INPUT_CNT
  movlw ErrorCount2
  movwf BAD_INPUT_CNT2
WRITE_OUTPUTS
;write the output to the pins
  movf OUTPUT, W							;load outputs to W
  movwf GPIO								;write values to GPIO
#IF MAXCHANNELS > 3
  movf OUTPUTC, W							;write ch4 outputs
  movwf PORTC
#ENDIF
  goto TIMELOOP

CHECK_CHANNELS
;Compare channels against PWM counter
;If the result is zero, the channel will be turned off
;Since the channels are turned on in the Main body, this will
;provide a reasonable PWM, since TMR0 is used to keep a consistent
;execution time for DOPWM
  incf PWM_CNTR, f							;increment the PWM counter

	;check ESC
  movf PWM_CNTR, W							;load PWM counter
  xorwf ESC_VAL, W							;check if the counters match
  btfsc STATUS, Z							;if they're equal
  bcf GPIO, 2								;set ESC output low

	;check Aileron
  movf PWM_CNTR, W							;load PWM counter
  xorwf AIL_VAL, W							;check if the counters match
  btfss STATUS, Z
  goto CHECK_ELE							;skip if not equal
  bcf GPIO, 0								;else zero the Aileron outputs
  bcf GPIO, 1

CHECK_ELE
	;check Elevator
  movf PWM_CNTR, W							;load PWM counter
  xorwf ELE_VAL, W							;check if the counters match
  btfss STATUS, Z
  goto NEXT									;skip if not equal
  bcf GPIO, 4								;else, zero the elevator outputs
  bcf GPIO, 5	

NEXT
#IF MAXCHANNELS > 3
  	;check Rudder
  movf PWM_CNTR, W							;load PWM counter
  xorwf RUD_VAL, W							;check if the counters match
  btfss STATUS, Z
  goto TIMELOOP									;skip if not equal
  bcf PORTC, 0								;else, zero the elevator outputs
  bcf PORTC, 1	
#ENDIF

TIMELOOP
	;This is to trace cycle time in MPLAB simulation, check the value of W after this statement
  movf TMR0, W

	;loop until TMR0:7 is zero
	;This will cause us to wait until no less than TIME_VAL usec have passed
	;here to insure a consistent minimum  duration is spent in DOPWM
	;Note that instructions before DOPWM is called will count in this value
  btfsc TMR0, 7
  goto TIMELOOP
	;set TMR0 back to TIME_VAL
  movlw TIME_VAL
  movwf TMR0

  retlw 0									;return to addr in stack, with 0 in W
;END DOPWM





READ_INPUT
;START SUBROUTINE READ_INPUT
	;read input, parsing out the values for certain channels
  call DOPWM
INPUT_WAIT_HIGH
	;loop until the input goes high
  btfss INPUT_PIN							;test input
  goto READ_INPUT							;if low, keep waiting

START_INPUT
	;start reading input
	;Initally look for a 3ms gap followed by a 300uS high period
  movlw WAIT_3mS + 1						;copy 3mS + 1(since we decfsz first) to nTimer
  movwf nTimer
  call DOPWM
LOOP_FOR_3mS
	;loop for 3mS, if the input goes low during this time
	;we must not have found the low period
  decf nTimer, f						;decrement nTimer
  btfsc STATUS, Z
  goto FOUND_LOW_PERIOD
  call DOPWM
  btfsc INPUT_PIN							;test input
  goto LOOP_FOR_3mS							;if still high, loop back
  goto INPUT_ERROR							;if low, jump to INPUT_ERROR

FOUND_LOW_PERIOD
	;try to find the start of the pulse train within the next 17mS
  call DOPWM
  movlw (WAIT_17mS/d'2')					;we call DOPWM twice in COUNT_START_PULSE, so divide by 2	
  movwf START_PULSE_LENGTH
  call DOPWM
COUNT_START_PULSE
	;count the remaining length of the high pulse
	;once it goes low, jump down to start reading channel 1
	;if the pulse is too long, it's an error
  btfss INPUT_PIN							;test input
  goto READ_SETUP							;if low, jump past
  call DOPWM
  btfss INPUT_PIN							;test input
  goto READ_SETUP							;if low, jump past
  call DOPWM
  decfsz START_PULSE_LENGTH, f				;decrement START_PULSE_LENGTH
  goto COUNT_START_PULSE					;if not zero, loop back
  goto INPUT_ERROR

	;start reading channels using this algorithm
	;start with a value 4 (or 6)
	;increment that for every loop it's low (- jumps to DOPWM)
	;if less than 10 (or 12), error
	;then add how long it stays high to the low count
	;if greater than 58, error

	;---prep to read first channel---
READ_SETUP
  movlw d'1'								;prep counter with 1
  movwf COUNTER
  call DOPWM								;**uncounted call 1
  movlw d'7'								;offset channel by number of calls to DoPWM 1,2,3 and +1 for first run
  movwf nTimer

READ_LOW
	;count how long the channel is low
  btfsc INPUT_PIN							;test input
  goto READ_CHECK_LOW						;if low, jump past
  call DOPWM
  incf nTimer, f							;increment nTimer
  goto READ_LOW
READ_CHECK_LOW
  call DOPWM								;**uncounted call 2
  movlw WAIT_300uS + 3 						;Load 300uS worth of cycles
  subwf nTimer, W							;subtract from nTimer
  btfss STATUS, C							;test
  goto INPUT_ERROR							;low was too short
  call DOPWM								;**uncounted call 3
READ_HIGH
  btfss INPUT_PIN							;test input
  goto READ_CHECK_HIGH						;if low, jump down
  call DOPWM
  incf nTimer, f							;else increment nTimer while the input is high
  goto READ_HIGH
READ_CHECK_HIGH
  call DOPWM								;**uncounted call 1A
  movlw MAX_ALLOWED_PULSE_WIDTH				;load max pulse width we believe is valid
  subwf nTimer, W							;subtract MAPW from nTimer
  btfsc STATUS, C							
  goto INPUT_ERROR							;if positive, nTimer was larger than the max pulse width, so it's an error
  call DOPWM								;**uncounted call 2A

	;choose where to save the channel
  movf  COUNTER, W
  addwf PCL, f								;increment program counter according to what channel we're parsing
  nop										;we don't use channel zero
  goto CHAN1
  goto CHAN2
  goto CHAN3
  goto CHAN4

CHAN1
  movf CH1_BUF_PTR, W						;load indirect addressing for channel 1
  goto SAVE_CHAN
CHAN2
  movf CH2_BUF_PTR, W						;load indirect addressing for channel 2
  goto SAVE_CHAN
CHAN3
  movf CH3_BUF_PTR, W						;load indirect addressing for channel 3
  goto SAVE_CHAN
CHAN4
  movf CH4_BUF_PTR, W						;load indirect addressing for channel 4
  goto SAVE_CHAN


SAVE_CHAN									;save the channel using indirect addressing to the register selected above
  movwf FSR
  movf nTimer, W
  movwf INDF	
  call DOPWM								;**uncounted call 3A				

	;prep for next channel
  movlw (d'3' + d'6')						;have to include uncounted calls to DOWPM 1A and 2A in addition to others
  movwf nTimer								;and pre-load nTimer
  movf COUNTER, w							;copy COUNTER to W, so we can test it after incrementing
  incf COUNTER, f							;increment the counter in it's location
  xorlw d'4'								;test the *PRE*-increment-counter against 4
  btfss STATUS, Z							;if equal, we're done, break out
  goto READ_LOW								;else, loop back and read another channel



	;done reading channels
  call DOPWM
  bcf INPUT_ERROR_FLAG						;reset the error flag
  retlw 0									;return with zero
;END READ_INPUT




INPUT_ERROR
	;Error reading input
	;Pin exceeded timeout or pulse width, etc
  bsf INPUT_ERROR_FLAG						;set a flag indicating inputs are not valid for use in DoPWM
  goto READ_INPUT							;and re-read inputs





SETUP
  clrf GPIO 								;clear outputs
#IFDEF __16F630
  clrf PORTC								;clear outputs
#ENDIF ;_16F630

#IFDEF __12C509A							;Configure for 12C509A
  movwf OSCCAL 								;config OSCCAL
  movlw b'00001000'							;set GP3 to input, GP0-2,4,5 as outputs
  tris GPIO
  movlw b'00001000'							;TMR0 config: no prescaler, internal clocking (Fosc/4 = 1 cycle)
  option
#ENDIF ;_12C509A

#IFNDEF __12C509A							;Configure for all other devices
  bsf STATUS, RP0							;Set config options in Bank1
  call 3FFh									
  movwf OSCCAL 								;config OSCCAL
  movlw b'00001000'							;TMR0 config: no prescaler, internal clocking (Fosc/4 = 1 cycle)
  movwf OPTION_REG
#IFDEF __12F675
  clrf ANSEL								;turn off the 12F675's A/D unit
#ENDIF ;_12F675
  movlw b'00001000'							;set GP3 to input, GP0-2,4,5 as outputs
  movwf TRISIO
#IFDEF __16F630								;config 16F630's PORTC as outputs
  movlw b'00000000'							
  movwf TRISC
#ENDIF ;_16F630
  bcf STATUS, RP0							;done with Bank1 config options
  movlw 07h									;Turn off Comparator unit (111)
  movwf CMCON
#ENDIF ;!12C509A

  movlw TIME_VAL							;load TMR0 with TIME_VAL 
  movwf TMR0

;initialize some values
  movlw ErrorCount1							;copy error values into counters
  movwf BAD_INPUT_CNT
  movlw ErrorCount2
  movwf BAD_INPUT_CNT2
  clrf nTimer
  clrf ESC_VAL
  clrf AIL_VAL
  clrf ELE_VAL
  clrf RUD_VAL

  movlw d'0'								;load the throttle with min for safety
  movwf ESC_BUF_OLD
  movlw CENTER_STICK						;center channels
  movwf AIL_BUF_OLD
  movwf ELE_BUF_OLD
  movwf RUD_BUF_OLD

	;clear outputs
  clrf PWM_CNTR
  clrf OUT_BUF
  clrf OUT_BUFC
  clrf OUTPUT								
  clrf OUTPUTC

#IFDEF JR
;Config for JR, Throttle on CH1
	;make indirect pointer to ESC_BUF
  movlw ESC_BUF
  movwf CH1_BUF_PTR
	;make indirect pointer to AIL_BUF
  movlw AIL_BUF
  movwf CH2_BUF_PTR
	;make indirect pointer to ELE_BUF
  movlw ELE_BUF
  movwf CH3_BUF_PTR
	;make indirect pointer to RUD_BUF
  movlw RUD_BUF
  movwf CH4_BUF_PTR
	;swap rudder and esc pointers for the pseudo-4channel
  #IFDEF FAKE4CH
     MESSG "***WARNING WARNING WARNING***   FAKE4CH ENABLED!!!! HOPE YOU KNOW WHAT YOU'RE DOING!"
     movlw RUD_BUF
     movwf CH1_BUF_PTR
     movlw ESC_BUF
     movwf CH4_BUF_PTR
  #ENDIF ;FAKE4CH
#ENDIF ;JR

#IFDEF FUTABA
;Config for Futaba/Hitec, Throttle on CH3
	;make indirect pointer to AIL_BUF
  movlw AIL_BUF
  movwf CH1_BUF_PTR
	;make indirect pointer to ELE_BUF
  movlw ELE_BUF
  movwf CH2_BUF_PTR
	;make indirect pointer to ESC_BUF
  movlw ESC_BUF
  movwf CH3_BUF_PTR
	;make indirect pointer to RUD_BUF
  movlw RUD_BUF
  movwf CH4_BUF_PTR
	;swap rudder and esc pointers for the pseudo-4channel
  #IFDEF FAKE4CH
     MESSG "***WARNING WARNING WARNING***   FAKE4CH ENABLED!!!! HOPE YOU KNOW WHAT YOU'RE DOING!"
     movlw RUD_BUF
     movwf CH3_BUF_PTR
     movlw ESC_BUF
     movwf CH4_BUF_PTR
  #ENDIF ;FAKE4CH
#ENDIF ;FUTABA

#IFDEF SANWA
;Config for SANWA, Throttle on CH2
	;make indirect pointer to AIL_BUF
  movlw AIL_BUF
  movwf CH1_BUF_PTR
	;make indirect pointer to ESC_BUF
  movlw ESC_BUF
  movwf CH2_BUF_PTR
	;make indirect pointer to ELE_BUF
  movlw ELE_BUF
  movwf CH3_BUF_PTR
	;make indirect pointer to RUD_BUF
  movlw RUD_BUF
  movwf CH4_BUF_PTR
	;swap rudder and esc pointers for the pseudo-4channel
  #IFDEF FAKE4CH
     MESSG "***WARNING WARNING WARNING***   FAKE4CH ENABLED!!!! HOPE YOU KNOW WHAT YOU'RE DOING!"
     movlw RUD_BUF
     movwf CH3_BUF_PTR
     movlw ESC_BUF
     movwf CH4_BUF_PTR
  #ENDIF ;FAKE4CH
#ENDIF ;SANWA

;done setup


MAIN
	;main body of the code
	;we loop in here forever
  call DOPWM
  call READ_INPUT

	;clear all the output pins to zero
  bcf OUT_BUF,0	;ail
  bcf OUT_BUF,1
  bcf OUT_BUF,2	;esc
  bcf OUT_BUF,4	;ele
  bcf OUT_BUF,5
#IF MAXCHANNELS > 3
  bcf OUT_BUFC, 0	;rud
  bcf OUT_BUFC, 1
#ENDIF

  call DOPWM

;calculate pin output values and pwm durations

#IFDEF CHAN4MIX
  ;Mix in the throttle setting
  #IFNDEF FAKE4CH  
    movf RUD_BUF, w
    addwf ESC_BUF, w	;add the throttle value
    movwf RUD_BUF
    movlw CENTER_STICK - MIX_OFFSET
    subwf RUD_BUF, w	;subtract neutral and offset
    btfss STATUS, C		;if C is set, the result was positive
    movlw 0				;else we went negative, set value to zero so we don't underflow
    movwf RUD_BUF
  #ELSE
    ;Under FAKE4CH mode, RUD and ESC is switched, so we have to here too...ugh.
    movf ESC_BUF, w
    addwf RUD_BUF, w	;add the throttle value
    movwf ESC_BUF
    movlw CENTER_STICK - MIX_OFFSET
    subwf ESC_BUF, w	;subtract neutral and offset
    btfss STATUS, C		;if C is set, the result was positive
    movlw 0				;else we went negative, set value to zero so we don't underflow
    movwf ESC_BUF
  #ENDIF ;FAKE4CH
#ENDIF ;CHAN4MIX

	;Calculate Aileron results
  movf AIL_BUF, w
  movwf currentVal
  movf AIL_BUF_OLD, w
  movwf lastVal
  call CALC_ACTUATOR
  btfsc pinVal, 0
  bsf AIL_LOW_PIN
  btfsc pinVal, 1
  bsf AIL_HIGH_PIN
  movf pwmVal, w
  movwf AIL_VAL
  movf lastVal, W
  movwf AIL_BUF_OLD

  call DOPWM

	;Calculate ESC results
  movf ESC_BUF, w
  movwf currentVal
  movf ESC_BUF_OLD, w
  movwf lastVal
  call CALC_ESC
  bcf ESC_PIN
  btfsc pinVal, 0
  bsf ESC_PIN
  movfw pwmVal
  movwf ESC_VAL
  movf lastVal, w
  movwf ESC_BUF_OLD

  call DOPWM

	;Calculate Elevator results
  movf ELE_BUF, w
  movwf currentVal
  movf ELE_BUF_OLD, w
  movwf lastVal
  call CALC_ACTUATOR
  btfsc pinVal, 0
  bsf ELE_LOW_PIN
  btfsc pinVal, 1
  bsf ELE_HIGH_PIN
  movf pwmVal, w
  movwf ELE_VAL
  movf lastVal, w
  movwf ELE_BUF_OLD

  call DOPWM

#IF MAXCHANNELS > 3
	;Calculate Rudder results
  movf RUD_BUF, w
  movwf currentVal
  movf RUD_BUF_OLD, w
  movwf lastVal
  #IFDEF CHAN4ESC
    call CALC_ESC		;treat as ESC
  #ELSE
    call CALC_ACTUATOR	;treat as actuator
  #ENDIF ;CHAN4ESC
  btfsc pinVal, 0
  bsf RUD_LOW_PIN
  btfsc pinVal, 1
  bsf RUD_HIGH_PIN
  movf pwmVal, w
  movwf RUD_VAL
  movf lastVal, w
  movwf RUD_BUF_OLD
#ENDIF ;MAXCHANNELS > 3

  call DOPWM


;done calculating output pins and PWM durations

;copy new output pin values to output register
  movf OUT_BUF, W	;load the outputs
  movwf OUTPUT		;move to output reg
#IF MAXCHANNELS > 3
  movf OUT_BUFC, W
  movwf OUTPUTC
#ENDIF

  GOTO MAIN
;END of main program, loop back





CALC_ACTUATOR
;Determine PIN values and DoPWM durations for actuator outputs
;Requires the following inputs and outputs
;currentVal IN
;lastVal IN/OUT
;pinVal OUT
;pwmVal OUT
;-----------------------------------

;The algorithm used for setting Actuator pins
; and pwm duration is as follows:

;Reset pin values in pinVal
;Average the old value and new value into currentVal (saving 
;a copy in lastVal for next time)
;If currentVal is CENTER_STICK +-1 , jump to end, leavings pins zeroed
;If currentVal < CENTER_STICK-1, shift it down by CENTER_STICK and complement
;	to get a positive inverted value
;If currentVal > CENTER_STICK+1, shift it down by CENTER_STICK to recenter, and set pins
;In all cases, pwmVal ends up being 2*currentVal
;pwmVal is the duration for which the pins should be held
;	at the designated values for PWM based control

;[MIN------CS-2][CS-1,CS,CS+1][CS+2-------MAX]
;    (one way)     dead^band    (other way)

  call DOPWM
  bcf pinVal, 0								;clear the pins
  bcf pinVal, 1
  call DOPWM
 
;Average the old value and new value for smoothing
  movf lastVal, W							;add the two values and store in currentVal
  addwf currentVal, f
  call DOPWM
  bcf STATUS, C  							;divide by two and store in currentVal
  rrf currentVal, f  						;shift right, same as dividing by 2

  call DOPWM
  movf currentVal, W 						;copy currentVal to lastVal
  movwf lastVal
  call DOPWM

	;test if currentVal is less than CENTER_STICK
  movlw CENTER_STICK - 1
  subwf currentVal, W
  btfsc STATUS, C							;test if negative or equal (cV<= CS-1, same as cV<CS)
  goto CHAN_HIGH							;positive, jump down (cV>CS-1, same as cV>=CS)

CHAN_LOW
	;set the pins and pwm duration for a value below CENTER_STICK
  call DOPWM
  bsf pinVal, 0								;set pin0 direction high
	;subtract CENTER_STICK + 1 and complement to get a positive invert value
  movlw CENTER_STICK + 1
  subwf currentVal, f
  comf currentVal, f						;complement f
  goto CHAN_END

CHAN_HIGH
	;chan is high, but is it higher than the deadband?
  call DOPWM
	;test if currentVal is less than CENTER_STICK + 1 (in deadband)
  movlw CENTER_STICK + 1
  subwf currentVal, W
  btfss STATUS, C							;(>=CS+2, same as > CS+1)
  goto CHAN_END								;(<CS+2, same as <=CS+1) in deadband

	;set pins and pwm duration for stick higher than CENTER_STICK
  call DOPWM
  bsf pinVal, 1								;set pin1 direction high
	;shift value down by CENTER_STICK and save to currentVal
  movlw CENTER_STICK
  subwf currentVal, f

CHAN_END
  call DOPWM
	;rotate left through carry, pad with 0 
	;this will double the value and save it to currentVal
  bcf STATUS, C
  rlf currentVal, W
  movwf pwmVal

  retlw 0
;END CALC_ACTUATOR




CALC_ESC
;set ESC_VAL and duration
;-------------------------
;Requires the following inputs and outputs
;currentVal OUT
;lastVal IN/OUT
;pinVal OUT
;pwmVal OUT
;The ESC is turned off for ESC_VAL less than MIN_THROTTLE_WIDTH
;For greater than MIN_THROTTLE_WIDTH, the output is turned on and and the duration
;is set to ESC_BUF-MIN_PULSE_WIDTH
  call DOPWM
	;Test for inverted throttle
  btfss OUT_BUF, 6
  goto ESC_NORMAL

	;For inverted throttle
  movlw MIN_THROTTLE_WIDTH + MAX_MEASURED_WIDTH
  subwf currentVal, f
  comf currentVal, f


ESC_NORMAL
  call DOPWM

	;Average the old value and new value
  movf lastVal, w
  addwf currentVal, f						;add values
  call DOPWM
  bcf STATUS, C
  rrf currentVal, f						;divide by two
  call DOPWM
	
  movf currentVal, w
  movwf lastVal								;copy currentVal to lastVal
  call DOPWM

	;subtract MIN_THROTTLE_WIDTH from currentVal
  movlw MIN_THROTTLE_WIDTH
  subwf currentVal, w
  btfsc STATUS, C						;test if the "positive flag" is set
  goto SET_ESC_HIGH						;greater than MIN_THROTTLE_WIDTH

	;less than/equal to MIN_THROTTLE_WIDTH, don't do PWM on the ESC, just turn it off
  call DOPWM
  bcf pinVal, 0						;set ESC output low
  retlw 0									;exit

SET_ESC_HIGH
	;ESC is greater than MIN_THROTTLE_WIDTH, set output pin high and calculate the
	;PWM duration by subtracting MIN_PULSE_WIDTH
  call DOPWM
  bsf pinVal, 0						;set ESC output high
  movlw MIN_PULSE_WIDTH
  subwf currentVal, w				;subtract MIN_PULSE_WIDTH from currentVal
  movwf pwmVal

  retlw 0
;END CALC_ESC

 END
_PULSE_WIDTH from currentVal
  movwf pwmVal

  retlw 0
;END CALC_ESC

 END
