;IR TX for FUTABA
;Originally from http://www.cityfujisawa.ne.jp/~toko/pic/
; Schematic available from above site
;See also http://www.rcgroups.com/forums/showthread.php?s=&threadid=24711
;
;Code for PIC12(C509A/F629/F675) @ 4MHZ with INT OSC
;
;GP 3 is input from Futaba FM TX buddy jack
;GP{0,1,2,4,5} are outputs	
;
;NOTE NOTE NOTE: 12F6XX has no weak pull up on GP3, so a ~33Kohm resistor to +Vcc is required
;
;Functional desc:
; IR Receiver module expects 57 khz modulation = 17.5 usec on and 17.5 usec off
; so the outputs must be turned on for ~17 operations, and off for ~17 operations
; with an accuracy of 17/17.5 (97.1 %)
; For every high pulse received, the TX will send a 500us modulated pulse


;Note: We can also modulate at 38khz, uncomment the following line
;#DEFINE f38K

;Note: JR radios have an inverted signal, uncomment the following for these radios
;#DEFINE INVERTED_SIGNAL




#IFDEF __12C509A
	MESSG "Assembling for 12C509A"
	LIST	P=PIC12C509A
	INCLUDE	"P12C509A.INC"
	__CONFIG _MCLRE_OFF & _CP_OFF & _WDT_OFF & _IntRC_OSC
	#DEFINE CBVAL 07H
#ENDIF

#IFDEF __12F629
	MESSG "Assembling for 12F629"
	LIST	P=PIC12F629
	INCLUDE "P12F629.INC"
	__CONFIG _MCLRE_OFF & _CP_OFF & _WDT_OFF & _INTRC_OSC_NOCLKOUT & _CPD_OFF & _PWRTE_ON & _BODEN_ON
	#DEFINE CBVAL 20H
#ENDIF

#IFDEF __12F675
	MESSG "Assembling for 12F675"
	LIST	P=PIC12F675
	INCLUDE "P12F675.INC"
	__CONFIG _MCLRE_OFF & _CP_OFF & _WDT_OFF & _INTRC_OSC_NOCLKOUT & _CPD_OFF & _PWRTE_ON & _BODEN_ON
	#DEFINE CBVAL 20H
#ENDIF


#IFDEF f38K
	#DEFINE NUM_LOOPS	D'20'	;need to loop 20 times for 38khz (20 * 26us = ~500 us)
#ELSE
	#DEFINE NUM_LOOPS	D'30'	;need to loop 30 times for 56.9khz (30 * 17us = ~510 us)
#ENDIF

 CBLOCK CBVAL
	;Variables
  COUNTER 									;Counter for 500 usec output 
 ENDC



 ORG 0
Main
  clrf GPIO									;set outputs LOW

#IFDEF __12C509A							;Configure for 12C509A
  movwf OSCCAL 								;config OSCCAL
  movlw b'10001000'							;Enable 12C509A's weak pull-up (WPU) on GP3
  option
  movlw b'00001000'							;set GP3 to input, GP0-2,4,5 as outputs
  tris GPIO
#ENDIF ;_12C509A


#IFNDEF __12C509A							;Configure for all other devices
  bsf     STATUS, RP0						;jump to BANK1, set BANK1 options
  call 3FFh									;have to fetch OSCCAL for 12FXXX units
  movwf OSCCAL 
#IFDEF __12F675
  clrf	ANSEL								;digital IO, shutoff the A/D unit
#ENDIF
  movlw b'00001000'
  movwf OPTION_REG							
  movlw b'00001000'
  movwf TRISIO 								;set GP3 input, others output
  bcf     STATUS, RP0						;reset to bank0
  movlw 07h									;Turn off Comparator unit (111)
  movwf CMCON
#ENDIF ;!12C509A


	;Loop, waiting for the input to go high.
WAIT_FOR_INPUT
#IFDEF INVERTED_SIGNAL
  btfsc GPIO, 3				;test input until it goes low
#ELSE
  btfss GPIO, 3				;test input until it goes high
#ENDIF
  goto WAIT_FOR_INPUT
  movlw NUM_LOOPS			;load LOOP's counter
  movwf COUNTER   


	;this loop will run for approx 500 us, turning the outputs on and off at ~50% duty cycle
	;The frequency corresponds to the desired frequency set
	;e.g. for 56.9khz (1/17.5us) the outputs are on for 8, and off for 9 (accuracy of 17/17.5 = 97%)
	;for 38khz (1/26.3us) the outputs are on for 13, and off for 13 (accuracy of 26/26.3 = 99%
LOOP					

	;set outputs HIGH for 8us (or 13us)
  movlw b'00110111'		
  movwf GPIO
  goto $+1
  goto $+1
  goto $+1
#IFDEF f38K
  goto $+1
  goto $+1
  nop
#ENDIF

	;set outputs LOW for 9us (or 13us)
  movlw b'00000000'		 
  movwf GPIO
  goto $+1
#IFDEF f38K
  goto $+1
  goto $+1
#ENDIF
  nop
	;these inst normally take 4 ops (decf=1, btfss=1[normally], goto LP1=2)
  decf COUNTER, F		;decrease counter and test for zero
  btfss STATUS, Z		;
  goto LOOP				;repeat if the counter was not zero

  goto WAIT_FOR_INPUT	;if zero, jump back to test input

  END
